export const version = () => '1.0.0';

export class Sensor {
    constructor(id, name, data = {}) {
      if (typeof id != 'string') {
        throw new Error('id n\'est pas du bon type');
      }
      this._id = id;
      if (typeof name != 'string') {
        throw new Error('name n\'est pas du bon type');
      }
      this._name = name;
      this.data = data; 
    }
    get id() {
      return this._id;
    }
    get name() {
      return this._name;
    }
    get data() {
      return this._data;
    }
    set data(dat) {
      if (typeof dat != 'object') {
        throw new Error('la donnée n\'est pas du bon type');
      }
      this._data = dat;
    }
    static toObj(object) {
      return new Sensor(object.id, object.name, object.data);
    }
    toString() {
      let str = `${this.name}@${this.id}`;
      return str;
    }
  }

  export class TimeSeries {
    constructor(values, labels) {
      if (!Array.isArray(values) || !Array.isArray(labels)) {
        throw new Error('TimeSeries accepte que arrays comme parametere');
      }
      this._values = values;
      this._labels = labels;
    }
  
    get values() {
      return this._values;
    }
    get labels() {
      return this._labels;
    }
  }

  export class Datum {
    constructor(value) {
      if (typeof value != 'number') {
        throw new Error('ça doit être un number');
      }
      this._value = value;
    }
    get value() {
      return this._value;
    }
    static toObj(object) {
      return new Datum(object.value);
    }
  }

  export class SensorTimeSeries extends Sensor {
    constructor(id, name, timeSeries) {
      timeSeries = new TimeSeries(timeSeries.values, timeSeries.labels);
      super(id, name, timeSeries);
    }
    set data(dat) {
      if (!(dat instanceof TimeSeries)) {
        throw new Error('la donnée n\'est pas du bon type');
      }
      this._data = dat;
    }
    get data() {
      return this._data;
    }
    static getFromSensor(sensor) {
      return new SensorTimeSeries(sensor.id, sensor.name, sensor.data);
    }
  
    toString() {
      return this.toStringWithSuffixe('');
    }
    static toObj(object) {
      return this.getFromSensor(Sensor.toObj(object));
    }
  
    toStringWithSuffixe(suffixe) {
      let str = `${this.name}@${this.id} {\n`;
      for (let i = 0; i < this.data.values.length; ++i) {
        str += `\t"${this.data.labels[i]}": ${this.data.values[i]}${suffixe},\n`;
      }
      return `${str}}`;
    }
  }

  export class SensorDatum extends Sensor {
    constructor(id, name, datum) {
      datum = new Datum(datum.value);
      super(id, name, datum);
    }
    set data(dat) {
      if (!(dat instanceof Datum)) {
        throw new Error('la donnée n\'est pas du bon type');
      }
      this._data = dat;
    }
    get data() {
      return this._data;
    }
    static toObj(object) {
      return this.getFromSensor(Sensor.toObj(object));
    }
    static getFromSensor(sensor) {
      return new SensorDatum(sensor.id, sensor.name, sensor.data);
    }
  }

  export class SensorTemperature extends SensorTimeSeries {
    static getFromSensor(sensor) {
      return new SensorTemperature(sensor.id, sensor.name, sensor.data);
    }
  
    toString() {
      return this.toStringWithSuffixe(' °C');
    }
  }

  export class SensorFan extends SensorTimeSeries {
    static getFromSensor(sensor) {
      return new SensorFan(sensor.id, sensor.name, sensor.data);
    }
  
    toString() {
      return this.toStringWithSuffixe(' rpm');
    }
  }

  export class SensorDoor extends SensorDatum {
    static getFromSensor(sensor) {
      return new SensorDoor(sensor.id, sensor.name, sensor.data);
    }
    get isDoorOpen() {
      return this.data.value == true; 
    }
  }

  export function getSensorFromObjectAuto(object) {
    switch (object.type) {
      case 'DOOR':
        return SensorDoor.toObj(object);
      case 'FAN_SPEED':
        return SensorFan.toObj(object);
      case 'TEMPERATURE':
        return SensorTemperature.toObj(object);
      default:
        throw new Error('object inconnue');
    }
  }